@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 info = {
     "name": "en",
     "date_order": "MDY",
@@ -182,6 +181,13 @@ info = {
         ],
         "2 day ago": [
             "day before yesterday"
+        ],
+        "1 decade ago": [
+            "last decade",
+            "this decade"
+        ],
+        "in 1 decade": [
+            "next decade"
         ]
     },
     "relative-type-regex": {
@@ -252,6 +258,12 @@ info = {
             "(\\d+) second ago",
             "(\\d+) seconds ago",
             "(\\d+) sec ago"
+        ],
+        "in \\1 decade": [
+            "in (\\d+) decades?"
+        ],
+        "\\1 decade ago": [
+            "(\\d+) decades? ago"
         ]
     },
     "locale_specific": {
@@ -754,6 +766,10 @@ info = {
         "of"
     ],
     "sentence_splitter_group": 1,
+    "decade": [
+        "decade",
+        "decades"
+    ],
     "ago": [
         "ago"
     ],
