@@ -6,17 +6,19 @@
 timezone_info_list = [
     {
         'regex_patterns':
-            [r'(.)%s$',],
+            [r'(.)%s$'],
         'replace':
             [
-             (r'(?:UTC|GMT)\\(\+|\-)0(\d):00', r'(?:UTC|GMT)\\\1\2'),   # UTC+n, UTC-n, GMT+n, GMT-n
-             (r'(?:UTC|GMT)\\(\+|\-)0(\d):(\d{2})', r'(?:UTC|GMT)\\\1\2:\3'),   # UTC+n:mm, UTC-n:mm,GMT+n:mm, GMT-n:mm
-             (r'(?:UTC|GMT)\\(\+|\-)(\d{2}):00', r'(?:UTC|GMT)\\\1\2'),     # UTC+nn, UTC-nn, GMT+nn, GMT-nn
-             (r'(?:UTC|GMT)(\\[+-])(\d{2}):(\d{2})', r'(?:UTC|GMT)\1\2:?\3.*'),     # UTC+nnmm, , UTC-nnmm, GMT+nnmm, GMT-nnmm
-             (r'UTC', r''),
-             (r':', r''),
-             (r':|UTC', r''),
-             (r'UTC', r'GMT'),
+                # UTC+n, UTC-n, GMT+n, GMT-n:
+                (r'(?:UTC|GMT)\\(\+|\-)0(\d):00', r'(?:UTC|GMT)\\\1\2'),
+                # UTC+n:mm, UTC-n:mm, GMT+n:mm, GMT-n:mm:
+                (r'(?:UTC|GMT)\\(\+|\-)0(\d):(\d{2})', r'(?:UTC|GMT)\\\1\2:\3'),
+                # UTC+nn, UTC-nn, GMT+nn, GMT-nn:
+                (r'(?:UTC|GMT)\\(\+|\-)(\d{2}):00', r'(?:UTC|GMT)\\\1\2'),
+                # UTC+nnmm, UTC-nnmm, GMT+nnmm, GMT-nnmm:
+                (r'(?:UTC|GMT)(\\[+-])(\d{2}):(\d{2})', r'(?:UTC|GMT)\1\2:?\3.*'),
+                # Others:
+                (r'UTC', r''), (r':', r''), (r':|UTC', r''), (r'UTC', r'GMT'),
             ],
         'timezones':
             [(r'UTC\-12:00', -43200),
